const {findAvailablePort} = require("../utilities/findAvailablePort");
const {input} = require("@inquirer/prompts");
const querystring = require("querystring");
const {nanoid} = require("nanoid");
const express = require('express')
const {spotify, spotifyCredentialsFile, hasSpotifyCredentials, saveSpotifyCredentials} = require("../utilities/spotify");

exports.command = 'authenticate'
exports.description = 'starts the Spotity authentication process'
exports.builder = (argv) => argv
    .option('client-id', {
        description: 'The Spotify app client id',
        type: 'string'
    })
    .option('client-secret', {
        description: 'The Spotify app client secret',
        type: 'string'
    })
exports.handler = async function (argv) {
    if (hasSpotifyCredentials()) {
        console.log(`spotify.local is already authenticated, remove "${spotifyCredentialsFile}" is you want to log out.`);
        return;
    }

    const port = await findAvailablePort();

    const authenticateRoute = 'authenticate';
    const redirectUri = `http://localhost:${port}/${authenticateRoute}`;

    const tutorial = [
        'Visit https://developer.spotify.com/dashboard and click "Create app"',
        'Set the "App name" as "spotify.local"',
        'Set the "App description" as "My spotify.local"',
        'Set the "Website" as "https://code.tmp.dev/ian/spotify.local"',
        `Set the "Redirect URI" as "${redirectUri}"`,
        'Select under "Which API/SDKs are you planning to use?" the "Web API"',
        'Sign your life away by agreeing with the "Developer Terms of Service and Design Guidelines"',
        'Click on the "Save" button',
        'Copy the "Client ID" from the settings page',
    ];

    tutorial.forEach((string, index) => console.log(`${index+1} ${string}`));

    const clientId = argv.clientId ?? await input({ message: `${tutorial.length}. Paste the client id here` })
    const clientSecret = argv.clientSecret ?? await input({ message: `${tutorial.length}. Paste the client secret here` })
    const originalState = nanoid();

    spotify.setClientId(clientId);
    spotify.setClientSecret(clientSecret);
    spotify.setRedirectURI(redirectUri);

    const authUrl = new URL('https://accounts.spotify.com/authorize');
    authUrl.search = querystring.stringify({
        response_type: 'code',
        client_id: spotify.getClientId(),
        scope: [
            'user-library-read',
            'user-read-private',
            'user-library-modify',
            'user-follow-modify',
            'user-follow-read',
            'playlist-read-private',
            'playlist-modify-public',
            'playlist-modify-private',
            'playlist-read-collaborative',
            'user-top-read',
            'user-read-recently-played',
            'user-read-playback-state',
            'user-modify-playback-state',
            'user-read-currently-playing',
            'user-read-playback-position'
        ].join(' '),
        redirect_uri: spotify.getRedirectURI(),
        state:  originalState
    });

    const app = express();

    let server;
    app.get(`/${authenticateRoute}`, async (req, res) => {
        const {state, code, error = null} = req.query;
        if (!error && originalState === state) {
            const { body } = await spotify.authorizationCodeGrant(code)
            const {access_token, refresh_token, expires_in} = body;
            spotify.setExpired((new Date).getTime() + expires_in);
            spotify.setAccessToken(access_token);
            spotify.setRefreshToken(refresh_token);
            saveSpotifyCredentials();

            res.send('<script>(() => close())()</script>');

            server?.close();

            console.log(`${tutorial.length+2}. Spotify.local has been authenticated`);

            return;
        }

        res.status(400);
        res.send('NO');
    });

    server = app.listen(port, () => console.log(`${tutorial.length+1}. Follow the following URL to authenticate: ${authUrl}`));
}