const net = require('net');

const startPort = 5000;
const endPort = 10000;

const portAvailable = (port) => new Promise((resolve, reject) => {
    const server = net.createServer();

    server.once('error', function(err) {
        if (err.code === 'EADDRINUSE') {
            reject([null, false])
        }

        reject([err])
    });

    server.once('listening', function() {
        server.close();
        resolve([null, true])
    });

    console.log(`Checking port ${port}`);

    server.listen(port);
});

/**
 * @returns {Promise<number>}
 */
exports.findAvailablePort = async function () {
    for (let port = startPort; startPort <= endPort; port++) {
        const [error, available] = await portAvailable(port);
        if (error) throw error;
        if (available) return port;
    }
}